#include "editor.h"

//{{{
void transform_mirror_horizontal(void)
{
	int top, bottom, left, right;
	int yi, leftpos, rightpos;
	int temp;
	
	get_selected_region(&top, &left, &right, &bottom);
	
	for(yi=top; yi<=bottom; yi++)
	{
		for(leftpos=left, rightpos=right;
		    rightpos>leftpos;
		    leftpos++, rightpos--)
		{
			temp = current_map->t[yi][leftpos];
			current_map->t[yi][leftpos]=mirror_x(current_map->t[yi][rightpos]);
			current_map->t[yi][rightpos]=mirror_x(temp);
		}
	}
}
//}}}
//{{{
void transform_mirror_vertical(void)
{
	int top, bottom, left, right;
	int xi, toppos, bottompos;
	int temp;
	
	get_selected_region(&top, &left, &right, &bottom);
	
	for(xi=left; xi<=right; xi++)
	{
		for(toppos=top, bottompos=bottom;
		    toppos < bottompos;
		    toppos++, bottompos--)
		{
			temp = current_map->t[toppos][xi];
			current_map->t[toppos][xi]=mirror_y(current_map->t[bottompos][xi]);
			current_map->t[bottompos][xi]=mirror_y(temp);
		}
	}
}
//}}}
//{{{
void transform_rotate_180(void)
{
	transform_mirror_horizontal();
	transform_mirror_vertical();
}
//}}}

//{{{
int mirror_x(int t)
{
	switch(t)
	{
		case TILE_SLIDER_E: return TILE_SLIDER_W;
		case TILE_SLIDER_W: return TILE_SLIDER_E;
		case TILE_ROCKY_E:   return TILE_ROCKY_W;
		case TILE_ROCKY_W:   return TILE_ROCKY_E;
		case TILE_TWIST_CW: return TILE_TWIST_CCW;
		case TILE_TWIST_CCW: return TILE_TWIST_CW;
		case TILE_PUSHER_W: return TILE_PUSHER_E;
		case TILE_PUSHER_E: return TILE_PUSHER_W;
		case TILE_WALL_1:   return TILE_WALL_3;
		case TILE_WALL_4:   return TILE_WALL_6;
		case TILE_WALL_7:   return TILE_WALL_9;
		case TILE_WALL_9:   return TILE_WALL_7;
		case TILE_WALL_6:   return TILE_WALL_4;
		case TILE_WALL_3:   return TILE_WALL_1;
		default: // For tiles without a known reflection, make no change
			return t;
	}
}
//}}}
//{{{
int mirror_y(int t)
{
	switch(t)
	{
		case TILE_SLIDER_N: return TILE_SLIDER_S;
		case TILE_SLIDER_S: return TILE_SLIDER_N;
		case TILE_ROCKY_N:   return TILE_ROCKY_S;
		case TILE_ROCKY_S:   return TILE_ROCKY_N;
		case TILE_TWIST_CW: return TILE_TWIST_CCW;
		case TILE_TWIST_CCW: return TILE_TWIST_CW;
		case TILE_PUSHER_S: return TILE_PUSHER_N;
		case TILE_PUSHER_N: return TILE_PUSHER_S;
		case TILE_WALL_1:   return TILE_WALL_7;
		case TILE_WALL_2:   return TILE_WALL_8;
		case TILE_WALL_3:   return TILE_WALL_9;
		case TILE_WALL_7:   return TILE_WALL_1;
		case TILE_WALL_8:   return TILE_WALL_2;
		case TILE_WALL_9:   return TILE_WALL_3;
		default: // For tiles without a known reflection, make no change
			return t;
	}
}
//}}}

//{{{
int tile_is_wall(int x, int y)
{
	if(!is_in_bounds(x,y)) return 0;
	switch(current_map->t[y][x])
	{
		case TILE_WALL_5: case TILE_WALL_1: case TILE_WALL_2:
		case TILE_WALL_3: case TILE_WALL_4: case TILE_WALL_6:
		case TILE_WALL_7: case TILE_WALL_8: case TILE_WALL_9:
			return 1;
		default:
			return 0;
	}
}
//}}}
//{{{
int wall_map[16] =
// Bit N in an index into wall_map corresponds to whether relative position N 
// contains a wall or not.
//     +---+
//     |   |
//     | 1 |
//     |   |
// +---+---+---+
// |   |###|   |
// | 0 |###| 2 |
// |   |###|   |
// +---+---+---+
//     |   |
//     | 3 |
//     |   |
//     +---+
	{
	// Position 0 clear
		// Position 1 clear
			// Position 2 clear
			TILE_WALL_5, // Position 3 clear
			TILE_WALL_8, // Position 3 set
			// Position 2 set
			TILE_WALL_4,  // Position 3 clear
			TILE_WALL_7,  // Position 3 set
		// Position 1 set
			// Position 2 clear
			TILE_WALL_2, // Position 3 clear
			TILE_WALL_5, // Position 3 set
			// Position 2 set
			TILE_WALL_1, // Position 3 clear
			TILE_WALL_5, // Position 3 set
	// Position 0 set
		// Position 1 clear
			// Position 2 clear
			TILE_WALL_6, // Position 3 clear
			TILE_WALL_9, // Position 3 set
			// Position 2 set
			TILE_WALL_5, // Position 3 clear
			TILE_WALL_5, // Position 3 set
		// Position 1 set
			// Position 2 clear
			TILE_WALL_3, // Position 3 clear
			TILE_WALL_5, // Position 3 set
			// Position 2 set
			TILE_WALL_5, // Position 3 clear
			TILE_WALL_5, // Position 3 set
	};
//}}}
//{{{
void transform_smooth(void)
{
	int top, bottom, left, right;
	int xi, yi;
	int dirs[4];
	
	get_selected_region(&top, &left, &right, &bottom);
	
	for(yi=top; yi<=bottom; yi++)
	for(xi=left; xi<=right; xi++)
	{
		if(!tile_is_wall(xi, yi))
			continue;
		dirs[0] = tile_is_wall(xi-1, yi);
		dirs[1] = tile_is_wall(xi, yi-1);
		dirs[2] = tile_is_wall(xi+1, yi);
		dirs[3] = tile_is_wall(xi, yi+1);
		current_map->t[yi][xi] =
			wall_map[ dirs[0]<<3 | dirs[1]<<2 | dirs[2]<<1 | dirs[3] ];
	}
}
//}}}

